<?php
/* --------------------------------------------------------------
   GXCoreLoader.inc.php 2016-03-31
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2016 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

MainFactory::load_class('GXCoreLoaderInterface');

/**
 * Class GXCoreLoader
 *
 * @category    System
 * @package     Loaders
 * @subpackage  GXCoreLoader
 */
class GXCoreLoader implements GXCoreLoaderInterface
{
	/**
	 * Contains the loader settings.
	 *
	 * @var GXCoreLoaderSettingsInterface
	 */
	protected $gxCoreLoaderSettings;
	
	/**
	 * Database Layer Object
	 *
	 * @var CI_DB_query_builder
	 */
	protected $ciDatabaseQueryBuilder;
	
	/**
	 * Factory for OrderService Objects
	 *
	 * @var AbstractOrderServiceFactory
	 */
	protected $orderServiceFactory;
	
	
	/**
	 * Class Constructor
	 *
	 * @param GXCoreLoaderSettingsInterface $gxCoreLoaderSettings
	 */
	public function __construct(GXCoreLoaderSettingsInterface $gxCoreLoaderSettings)
	{
		$this->gxCoreLoaderSettings = $gxCoreLoaderSettings;
	}
	
	
	/**
	 * Get the requested server object.
	 *
	 * @param string $serviceName
	 *
	 * @return AddressBookServiceInterface|CountryServiceInterface|CustomerServiceInterface
	 *
	 * @throws DomainException
	 *
	 * @todo Delegate to GXServiceFactory
	 */
	public function getService($serviceName)
	{
		switch($serviceName)
		{
			case 'Customer': // DEPRECATED!!
				$customerServiceFactory = $this->_getCustomerServiceFactory();
				$customerService        = $customerServiceFactory->getCustomerService();
				
				return $customerService;
			
			case 'CustomerRead':
				$customerServiceFactory = $this->_getCustomerServiceFactory();
				$customerReadService    = $customerServiceFactory->createCustomerReadService();
				
				return $customerReadService;
			
			case 'CustomerWrite':
				$customerServiceFactory = $this->_getCustomerServiceFactory();
				$customerWriteService   = $customerServiceFactory->createCustomerWriteService();
				
				return $customerWriteService;
			
			case 'AddressBook':
				$customerServiceFactory = $this->_getCustomerServiceFactory();
				$addressBookService     = $customerServiceFactory->getAddressBookService();
				
				return $addressBookService;
			
			case 'Country':
				$customerServiceFactory = $this->_getCustomerServiceFactory();
				$countryService         = $customerServiceFactory->getCountryService();
				
				return $countryService;
			
			case 'RegistrationInputValidator':
				$customerServiceFactory     = $this->_getCustomerServiceFactory();
				$registrationInputValidator = $customerServiceFactory->getCustomerRegistrationInputValidatorService();
				
				return $registrationInputValidator;
			
			case 'AccountInputValidator':
				$customerServiceFactory = $this->_getCustomerServiceFactory();
				$accountInputValidator  = $customerServiceFactory->getCustomerAccountInputValidator();
				
				return $accountInputValidator;
			
			case 'AddressInputValidator':
				$customerServiceFactory = $this->_getCustomerServiceFactory();
				$accountInputValidator  = $customerServiceFactory->getCustomerAddressInputValidatorService();
				
				return $accountInputValidator;
			
			case 'UserConfiguration':
				$db                       = $this->getDatabaseQueryBuilder();
				$userConfigurationReader  = MainFactory::create('UserConfigurationReader', $db);
				$userConfigurationWriter  = MainFactory::create('UserConfigurationWriter', $db);
				$userConfigurationService = MainFactory::create('UserConfigurationService', $userConfigurationReader,
				                                                $userConfigurationWriter);
				
				return $userConfigurationService;
			
			case 'Statistics':
				$db                = $this->getDatabaseQueryBuilder();
				$xtcPrice          = new xtcPrice($_SESSION['currency'],
				                                  $_SESSION['customers_status']['customers_status_id']);
				$statisticsService = MainFactory::create('StatisticsService', $db, $xtcPrice);
				
				return $statisticsService;
			
			case 'Email':
				$emailFactory = $this->_getEmailFactory();
				
				return $emailFactory->createService();
			
			case 'OrderObject':
				$factory = $this->_getOrderServiceFactory();
				
				return $factory->createOrderObjectService();
			
			case 'OrderRead':
				$factory = $this->_getOrderServiceFactory();
				
				return $factory->createOrderReadService();
			
			case 'OrderWrite':
				$factory = $this->_getOrderServiceFactory();
				
				return $factory->createOrderWriteService();
			
			case 'Http':
				$httpServiceFactory = MainFactory::create('HttpServiceFactory');
				
				return $httpServiceFactory->createService();
			
			case 'CategoryRead':
				$factory = $this->_getCategoryServiceFactory();
				
				return $factory->createCategoryReadService();
			
			case 'CategoryWrite':
				$factory = $this->_getCategoryServiceFactory();
				
				return $factory->createCategoryWriteService();
			
			case 'CategoryObject':
				$factory = $this->_getCategoryServiceFactory();
				
				return $factory->createCategoryObjectService();
			
			case 'AddonValue':
				$factory = $this->_getAddonValueServiceFactory();
				
				return $factory->createAddonValueService();
			
			case 'ProductRead':
				$factory = $this->_getProductServiceFactory();
				
				return $factory->createProductReadService();
			
			case 'ProductWrite':
				$factory = $this->_getProductServiceFactory();
				
				return $factory->createProductWriteService();
			
			case 'ProductObject':
				$factory = $this->_getProductServiceFactory();
				
				return $factory->createProductObjectService();
			
			default:
				throw new DomainException('Unknown service: ' . htmlentities($serviceName));
		}
	}
	
	
	/**
	 * Method depends on CodeIgniter database library
	 *
	 * @return CI_DB_query_builder
	 *
	 * @todo check connection errors
	 * @todo escape special characters in mysqli connection string (AT)
	 * @todo use GXDatabaseAccessorInterface
	 */
	public function getDatabaseQueryBuilder()
	{
		if($this->ciDatabaseQueryBuilder == null)
		{
			$dbUser     = $this->gxCoreLoaderSettings->getDatabaseUser();
			$dbPassword = $this->gxCoreLoaderSettings->getDatabasePassword();
			$dbServer   = $this->gxCoreLoaderSettings->getDatabaseServer();
			$dbName     = $this->gxCoreLoaderSettings->getDatabaseName();
			$dbSocket   = ($this->gxCoreLoaderSettings->getDatabaseSocket()) ? '?socket='
			                                                                   . $this->gxCoreLoaderSettings->getDatabaseSocket() : '';
			
			$connectionString = 'mysqli://' . $dbUser . ':' . $dbPassword . '@' . $dbServer . '/' . $dbName . $dbSocket;
			
			$this->ciDatabaseQueryBuilder = CIDB($connectionString);
			
			// @todo Remove the following block when the shop is totally ready for MySQL strict mode.
			if(is_object($GLOBALS['coo_debugger'])
			   && $GLOBALS['coo_debugger']->is_enabled('enable_mysql_strict_mode')
			)
			{
				$this->ciDatabaseQueryBuilder->query('SET SESSION sql_mode = "ONLY_FULL_GROUP_BY,STRICT_TRANS_TABLES,'
				                                     . 'NO_ZERO_IN_DATE,NO_ZERO_DATE,ERROR_FOR_DIVISION_BY_ZERO,'
				                                     . 'NO_AUTO_CREATE_USER,NO_ENGINE_SUBSTITUTION"');
			}
			else
			{
				$this->ciDatabaseQueryBuilder->query('SET SESSION sql_mode = ""');
			}
		}
		
		return $this->ciDatabaseQueryBuilder;
	}
	
	
	/**
	 * Get a customer service factory object.
	 *
	 * @return CustomerServiceFactory
	 */
	protected function _getCustomerServiceFactory()
	{
		$ciDatabaseQueryBuilder = $this->getDatabaseQueryBuilder();
		$customerServiceFactory = MainFactory::create('CustomerServiceFactory', $ciDatabaseQueryBuilder);
		
		return $customerServiceFactory;
	}
	
	
	/**
	 * Get an email factory object.
	 *
	 * @return EmailFactory
	 */
	protected function _getEmailFactory()
	{
		$db           = $this->getDatabaseQueryBuilder();
		$emailFactory = MainFactory::create('EmailFactory', $db);
		
		return $emailFactory;
	}
	
	
	/**
	 * Get an order service factory object.
	 *
	 * @return AbstractOrderServiceFactory
	 */
	protected function _getOrderServiceFactory()
	{
		if($this->orderServiceFactory === null)
		{
			$db                        = $this->getDatabaseQueryBuilder();
			$this->orderServiceFactory = MainFactory::create('OrderServiceFactory', $db);
		}
		
		return $this->orderServiceFactory;
	}
	
	
	/**
	 * Get a category service factory.
	 *
	 * @return CategoryServiceFactory
	 */
	protected function _getCategoryServiceFactory()
	{
		$db                     = $this->getDatabaseQueryBuilder();
		$settings               = MainFactory::create('EnvCategoryServiceSettings');
		$categoryServiceFactory = MainFactory::create('CategoryServiceFactory', $db, $settings);
		
		return $categoryServiceFactory;
	}
	
	
	/**
	 * Get a product service factory
	 *
	 * @return ProductServiceFactory
	 */
	protected function _getProductServiceFactory()
	{
		$db                    = $this->getDatabaseQueryBuilder();
		$productServiceFactory = MainFactory::create('ProductServiceFactory', $db);
		
		return $productServiceFactory;
	}
	
	
	/**
	 * Get an addon value service factory.
	 *
	 * @return AddonValueServiceFactory
	 */
	public function _getAddonValueServiceFactory()
	{
		$db                       = $this->getDatabaseQueryBuilder();
		$addonValueServiceFactory = MainFactory::create('AddonValueServiceFactory', $db);
		
		return $addonValueServiceFactory;
	}
}
